﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ProgrammezLucene.Indexer
{
    /// <summary>
    /// Classe permettant d'indexer la base AdventureWorks
    /// </summary>
    internal class AdventureWorksIndexer
    {
        /// <summary>
        /// Démarre l'indexation
        /// </summary>
        internal void Index()
        {
            this.IndexProducts();
        }

        /// <summary>
        /// Index les produits
        /// </summary>
        private void IndexProducts()
        {
            //Nombre d'éléments maximum à récupérer (Utile uniquement pour les perfs sur les bases volumineuses)
            const int ITEMMAXTOGET = 100000;
            int i = 0;
            int j = 0;

            List<Data.Product> query = null;

            //On créé l'analyseur
            Lucene.Net.Analysis.Standard.StandardAnalyzer analyser = new Lucene.Net.Analysis.Standard.StandardAnalyzer(Lucene.Net.Util.Version.LUCENE_30);

            //On démarre l'écriture de l'index
            using (Lucene.Net.Index.IndexWriter writer = new Lucene.Net.Index.IndexWriter(Lucene.Net.Store.FSDirectory.Open(@"c:\temp\Lucene\AdventureWorks\Index"), analyser, true, Lucene.Net.Index.IndexWriter.MaxFieldLength.LIMITED))
            {
                //On compte le nombre de résultats en base de données
                Data.AdventureWorksEntities db = new Data.AdventureWorksEntities();
                var count = (from p in db.Product select p).Count();
                db.Dispose();

                while (j * ITEMMAXTOGET < count)
                {
                    using (db = new Data.AdventureWorksEntities())
                    {
                        query = null;
                        Console.WriteLine("Getting Data...");
                        // On récupère les produits par lot
                        query = (from p in db.Product select p).OrderBy(p => p.ProductID).Skip(j * ITEMMAXTOGET).Take(ITEMMAXTOGET).ToList();
                        j++;

                        Console.WriteLine("Indexing products...");
                        for (int k = 0; k < query.Count(); k++)
                        {
                            Data.Product item = query[k];
                            i++;

                            //On crée le document
                            Lucene.Net.Documents.Document doc = new Lucene.Net.Documents.Document();

                            // On crée les champs
                            Lucene.Net.Documents.Field field1 = new Lucene.Net.Documents.Field("ProductID", item.ProductID.ToString(), Lucene.Net.Documents.Field.Store.YES, Lucene.Net.Documents.Field.Index.NOT_ANALYZED);
                            Lucene.Net.Documents.Field field2 = new Lucene.Net.Documents.Field("ListPrice", item.ListPrice.ToString(), Lucene.Net.Documents.Field.Store.YES, Lucene.Net.Documents.Field.Index.NO);
                            Lucene.Net.Documents.Field field3 = new Lucene.Net.Documents.Field("ListPriceForSort", item.ListPrice.ToString(), Lucene.Net.Documents.Field.Store.NO, Lucene.Net.Documents.Field.Index.NOT_ANALYZED);
                            Lucene.Net.Documents.Field field4 = new Lucene.Net.Documents.Field("Name", item.Name, Lucene.Net.Documents.Field.Store.YES, Lucene.Net.Documents.Field.Index.ANALYZED);
                            Lucene.Net.Documents.Field field5 = new Lucene.Net.Documents.Field("NameForSort", item.Name, Lucene.Net.Documents.Field.Store.NO, Lucene.Net.Documents.Field.Index.NOT_ANALYZED);

                            //On ajoute les champs au document
                            doc.Add(field1);
                            doc.Add(field2);
                            doc.Add(field3);
                            doc.Add(field4);
                            doc.Add(field5);

                            //On ajoute le document à l'index
                            writer.AddDocument(doc);

                            //On optimise les performances
                            writer.Optimize();
                            Console.WriteLine(i.ToString() + " items indexed / " + count.ToString() + " items");
                            item = null;
                        }

                        // On libère les ressources
                        writer.Dispose();
                    }
                }
            }
        }
    }
}
