﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace ProgrammezLucene.Search
{
    /// <summary>
    /// Programme d'exemple pour faire une recherche
    /// </summary>
    class Program
    {
        /// <summary>
        /// Point d'entrée du programme d'exemple
        /// </summary>
        /// <param name="args">Arguments passés au programme</param>
        static void Main(string[] args)
        {
            //Recherche que l'on désire faire
            string search1 = "mountain";
            // Création de l'analyseur
            var analyzer = new Lucene.Net.Analysis.Standard.StandardAnalyzer(Lucene.Net.Util.Version.LUCENE_30);
            // Création de l'indexSearcher sur le répertoire utilisé par l'indexeur
            var searcher = new Lucene.Net.Search.IndexSearcher(Lucene.Net.Store.FSDirectory.Open(@"c:\temp\Lucene\AdventureWorks\Index"));

            //Recherche sur le champ Name
            Lucene.Net.Index.Term term = new Lucene.Net.Index.Term("Name", search1);
            // Requete de recherche
            Lucene.Net.Search.Query query1 = new Lucene.Net.Search.TermQuery(term);
            //Démarre la recherche
            Search("Searching mountain with TermQuery:", query1, searcher);
            // Libère les ressources
            searcher.Dispose();
            Console.ReadLine();
        }

        /// <summary>
        /// Permet de faire une recherche dans la Lucene
        /// </summary>
        /// <param name="title">Titre de la recherche</param>
        /// <param name="query">Requete de recherche</param>
        /// <param name="searcher">IndexSearcher pour réaliser la recherche</param>
        private static void Search(string title, Lucene.Net.Search.Query query, Lucene.Net.Search.IndexSearcher searcher)
        {
            Console.WriteLine(title);
            //Démarre la recherche
            Lucene.Net.Search.TopDocs hits = searcher.Search(query, null, 1000, Lucene.Net.Search.Sort.RELEVANCE);

            Console.WriteLine(hits.TotalHits + " result(s).");
            // Affiche les 1000 premièrs résultats (s'il y a 1000 résultats)
            for (int i = 0; i < hits.TotalHits && i < 1000; i++)
            {
                // Récupère le document par son index
                Lucene.Net.Documents.Document doc = searcher.Doc(hits.ScoreDocs[i].Doc);
                // Récupère les informations du produit
                string name = doc.Get("Name");
                string productID = doc.Get("ProductID");
                string listPrice = doc.Get("ListPrice");
                Console.WriteLine(string.Format("{0} - {1} : {2}", productID, name, listPrice));
            }
        }
    }
}
